package com.supervision.util;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.dto.paddlespeech.res.TtsResultDTO;
import lombok.extern.slf4j.Slf4j;

import java.util.*;

/**
 * 预定义答案助手
 */
@Slf4j
public class PredefinedAnswerAssistant {

    /**
     * 问候答案
     */
    private static final Map<String,TtsResultDTO> helloAnswerMap = new HashMap<>();

    /**
     * 预定义答案
     */
    private static final Map<String,TtsResultDTO> predefinedAnswerMap = new HashMap<>();

    /**
     * 阈值时间，超过这个阈值就开始调换出一个预置回答 单位毫秒 默认1000毫秒
     */
    private int thresholdMillis = 500;

    /**
     * 初始化时间
     */
    private DateTime initTime = DateUtil.date().offset(DateField.MILLISECOND, thresholdMillis);

    private boolean skip = false;

    private boolean skipFirst = false;


    /**
     * 当前时间是否超过阈值
     * @return true 超过阈值 false 未超过阈值
     */
    public boolean overThreshold(){

        return !skip && initTime.before(new Date());
    }

    public void skipFirst(){
        this.skipFirst = true;
    }

    public boolean isSkipFirst(){
        return skipFirst;
    }

    /**
     * 获取问候答案
     * @return 问候答案
     */
    public String getHelloAnswer() {
        List<String> collect = helloAnswerMap.keySet().stream().toList();
        return collect.get(RandomUtil.randomInt(0, collect.size() - 1));
    }

    public void setThresholdMillis(int thresholdMillis) {
        this.thresholdMillis = thresholdMillis;
    }

    public void closePreviewAnswer(){
        skip = true;
    }

    /**
     * 获取问候答案音频
     * @param text 问候答案
     * @return 问候答案音频
     */
    public String getHelloAnswerAudio(String text) {
        return helloAnswerMap.get(text).getAudio();
    }



    /**
     * 获取预定义答案
     * @return 预定义答案
     */
    public String getPredefinedAnswer() {
        List<String> collect = predefinedAnswerMap.keySet().stream().toList();
        return collect.get(RandomUtil.randomInt(0, collect.size() - 1));
    }

    public String getPredefinedAnswerAudio(String text) {
        return predefinedAnswerMap.get(text).getAudio();
    }

    public void resetInitTime(String text){
        if (StrUtil.isEmpty(text)){
            return;
        }
        TtsResultDTO ttsResultDTO = null == predefinedAnswerMap.get(text) ? helloAnswerMap.get(text) : predefinedAnswerMap.get(text);
        if (null == ttsResultDTO){
            return;
        }
        double v = NumberUtil.parseDouble(ttsResultDTO.getDuration()) * 1000;
        this.initTime = DateUtil.offsetMillisecond(DateUtil.date(), NumberUtil.parseInt(String.valueOf(v)));
    }


    /**
     * 加载问候答案
     */
    public void loadAnswer() {

        List<String> helloAnswerList = List.of("您好，我是您的康养顾问小苏，很开心接到您的来电","您好，我是您的康养顾问小苏，感谢您的来电",
                "您好，我是您的康养顾问小陈，很高兴接到您的电话");

        log.info("开始加载问候答案....");
        for (String text : helloAnswerList) {
            if (helloAnswerMap.containsKey(text)) {
                continue;
            }
            TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(text);
            helloAnswerMap.put(text,ttsResultDTO);
        }
        log.info("问候答案加载完成....");

        List<String> predefinedAnswerList = List.of("您好，我是您的康养顾问小陈，很高兴接到您的电话", "我会结合最新的资料，为您提供最贴心的解答",
                "结合最新的信息，为您提供最合适的建议", "结合最新的资料为您提供详细的解答", "根据掌握的高黎贡勐赫小镇信息，为您提供最准确的服务与建议",
                "根据最新动态，为您提供最精准的解答与建议", "结合最新情况，为您提供最贴心的服务与支持");

        log.info("开始加载预定义答案....");
        for (String text : predefinedAnswerList) {
            if (predefinedAnswerMap.containsKey(text)){
                continue;
            }
            TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(text);
            predefinedAnswerMap.put(text,ttsResultDTO);
        }
        log.info("预定义答案加载完成....");
    }


}
